# Copyright (C) 2008-2010 eBox Technologies S.L.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License, version 2, as
# published by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package EBox::Config;

use EBox::Exceptions::External;
use EBox::Gettext;

my $ref = {};
$ref->{prefix} = "/usr";
$ref->{datadir} = "/usr/share";
$ref->{sysconfdir} = "/etc";
$ref->{localstatedir} = "/var";
$ref->{libdir} = "/usr/lib";

for my $key (keys(%{$ref})) {
    if (substr($ref->{$key},-1) ne '/') {
        $ref->{$key} = ($ref->{$key} . "/");
    }
}

$ref->{package} = "libebox";
$ref->{version} = "2.0.8";
$ref->{perlpath} = "/usr/share/perl5/";

sub etc
{
    return $ref->{sysconfdir} . "ebox/";
}

sub var
{
    return $ref->{localstatedir};
}

# Function: configkeyFromFile
#
#      Get a configuration key from the specified file
#
# Parameters:
#
#      key - String the configuration key
#      file - String the configuration file path
#
# Returns:
#
#      String - the configuration value for that key if found
#
#      undef - if the configuration key is not in the configuration file
#
# Exceptions:
#
#      <EBox::Exceptions::External> - thrown if the file cannot be
#      opened
#
sub configkeyFromFile # (key, file)
{
    my ($key, $file) = @_;
    my $value = undef;
    open(my $CONF, $file) or
        throw EBox::Exceptions::External(
                __x('Could not open the config file {file}.', file => $file));
    my @lines = <$CONF>;
    for my $line (@lines) {
        $line =~ s/\s//g;
        $line =~ s/#.*//g;
        ($line eq '') and next;
        my ($nkey, $nvalue) = $line =~ /(.*?)=(.*)/;
        if ($nkey eq $key) {
            $value = $nvalue;
            last;
        }
    }
    close($CONF);
    return $value;
}

sub configkey # (key)
{
    my $key = shift;
    my $value = undef;

    my @conffiles = glob(EBox::Config::etc() . "[0-9][0-9]*.conf");

    foreach my $file (@conffiles) {
        $value = configkeyFromFile($key, $file);
        defined($value) and last;
    }
    if (defined($value)) {
        return $value;
    } else {
        return '';
    }
}

sub configkeys # (key)
{
    my $key = shift;

    my @conffiles = glob(EBox::Config::etc() . "[0-9][0-9]*.conf");

    my @values;
    foreach my $file (@conffiles){
        my $value = configkeyFromFile($key, $file);
        push (@values, $value) if defined($value);
    }
    return \@values;
}

# Function: configKeysFromFile
#
#      Get all configuration keys from a file
#
# Parameters:
#
#      file - String the configuration file path
#
# Returns:
#
#      hash ref - the keys with their values as it follows:
#
#         key - String the key name
#         value - String the value for that key
#
# Exceptions:
#
#      <EBox::Exceptions::DataNotFound> - thrown if the file cannot be
#      opened
#
sub configKeysFromFile # (file)
{
    my ($file) = @_;

    my $value = undef;
    open(my $CONF, $file) or
        throw EBox::Exceptions::DataNotFound(data => 'config file',
                value => $file);
    my %configKeys = ();
    while (<$CONF>) {
        $_ =~ s/\s//g;
        $_ =~ s/#.*//g;
        ($_ eq '') and next;
        my ($nkey, $nvalue) = $_ =~ /(.*)=(.*)/;
# Support for several values for a single key if comma is set
        if ( $nvalue =~ m/,/g ) {
            my @values = split( ',', $nvalue);
            $configKeys{$nkey} = \@values;
        } else {
            $configKeys{$nkey} = $nvalue;
        }
    }
    close($CONF);
    return \%configKeys;
}

sub user
{
    my $user = configkey('user');
    $user or throw EBox::Exceptions::External(
            __('The ebox user has not been set in the config file.'));
    return $user;
}

sub groups
{
    my @values = @{configkeys('group')};
    @values or throw EBox::Exceptions::External("The ebox group has not ".
            "been set in the config file.");
    my @groups;
    for my $group (@values) {
        if ($group =~ m/:/) {
            push (@groups, split(':', $group));
        } else {
            push (@groups, $group);
        }
    }

    return \@groups;
}

sub group
{
    my $value = configkey('egroup');
    $value or throw EBox::Exceptions::External("The ebox group has not ".
            "been set in the config file.");
    return $value;
}

sub home
{
    my $user = user();
    my ($name,$passwd,$uid,$gid, $quota,$comment,$gcos,$dir,$shell,$expire) = getpwnam($user);
    return $dir;
}

sub prefix
{
    return $ref->{prefix};
}

sub share
{
    return $ref->{datadir};
}

sub lib
{
    return $ref->{libdir};
}

sub libexec
{
    return $ref->{libdir} . "ebox/";
}

sub pkgdata
{
    return $ref->{datadir} . "ebox/";
}

sub locale
{
    return $ref->{datadir} . "locale/";
}

sub conf
{
    return $ref->{localstatedir} . "lib/ebox/conf/";
}

sub tmp
{
    return $ref->{localstatedir} . "lib/ebox/tmp/";
}

# Method: downloads
#
#      Get the path of the directory where the files to be downloaded are put
#
# Returns:
#
#      String - the path to that directory
sub downloads
{

    return tmp() . 'downloads/';
}

sub passwd
{
    return $ref->{localstatedir} . "lib/ebox/conf/ebox.passwd";
}


# Method: sessionid
#
#      Get the path where Web session identifier is stored
#
# Returns:
#
#      String - the path to that file

sub sessionid
{
    return $ref->{localstatedir} . 'lib/ebox/conf/ebox.sid';
}

# Method: scriptSession
#
#      Get the path where the script session identifier is stored
#
# Returns:
#
#      String - the path to that file
#
sub scriptSession
{
    return $ref->{localstatedir} . 'lib/ebox/conf/ebox.script-sid';
}

sub log
{
    return $ref->{localstatedir} . "log/ebox/";
}

sub logfile
{
    return $ref->{localstatedir} . "log/ebox/ebox.log";
}

sub stubs
{
    return $ref->{datadir} . "ebox/stubs/";
}

sub cgi
{
    return $ref->{datadir} . "ebox/cgi/";
}

sub templates
{
    return $ref->{datadir} . "ebox/templates/";
}

sub schemas
{
    return $ref->{datadir} . "ebox/schemas/";
}

sub www
{
    return $ref->{datadir} . "ebox/www/";
}

sub css
{
    return $ref->{datadir} . "ebox/www/css/";
}

sub images
{
    return $ref->{datadir} . "ebox/www/images/";
}


sub dynamicwww
{
    return $ref->{localstatedir} . 'lib/ebox/dynamicwww/';
}

sub dynamicwwwSubdirs
{
    return [ dynamicimages(), dynamicRSS() ];
}

sub dynamicimages
{
    return dynamicwww() . 'images/';
}

sub dynamicRSS
{
    return dynamicwww() . 'feed/';
}

sub package
{
    return $ref->{package};
}

sub version
{
    return $ref->{version};
}

sub lang
{
    return $ref->{lang};
}

# Method: perlPath
#
#      Get the PERL path where the perl modules lying on (Static
#      method).
#
# Returns:
#
#      String - the perl path
#
sub perlPath
{
    return $ref->{perlpath};
}

1;
